C#######################################################################
C  File FinDifTT.f
C  Finite Differences Travel Time Map
C  Author: Mihai Popovici (Jos Van Trier's algorithm)
C  Modified: Dave Nichols added cflfudge parameter.
C
C The CFL (Courant, Friedrichs, Lewy) stability condition JUN/91
C implemented after Dave Hale's code
C
C x_shot                  = shot X coordinate
C z_shot                  = shot Z coordinate
C nz                      = number of depth levels on the grid
C o_x,d_x                 = left X coordinate on the grid and increment
C nx                      = number of X points on the grid
C o_z,d_z                 = first depth level and depth increments
C ntheta                  = number of angles for the grid
C nr                      = number of radius increments
C slow(nz,nx)             = slowness model
C travel_time(nz,nx)      = output travel time map
C cflfudge      	  = Fudge factor for CFL condition
C			  1=apply strictly >1 == less strict
C 
C When  raytracing is unstable increase the ratio nr/ntheta
C (Increase nr or decrease ntheta).
C
C#######################################################################
        subroutine FDTraTim(x_shot,z_shot,nx,o_x,d_x,nz,o_z,d_z,
     &  ntheta,nr,slow,travel_time,t,u,w,fx,slow_rad,slowtemp,ttemp,
     &  nxo,nzo,oxo,dxo,ozo,dzo,cflfudge)
	implicit none
        integer nx,nz,nr,ntheta,nxo,nzo
        real u(0:ntheta+1,2),w(0:ntheta+1,2)
        real slow(nz,nx), travel_time(nzo,nxo)
        real t(0:ntheta+1,nr), slow_rad(ntheta+2,nr)
        real fx(0:ntheta+1)
        real slowtemp(ntheta+2),ttemp(0:ntheta+1) 
        real x_shot,z_shot,o_x,d_x,o_z,d_z,otheta,dtheta,o_radius
        real oxo,ozo,dxo,dzo,cflfudge
        real dr,rdr,rdt,ddt,PI,PI2
        real eps,epsilon,xmax,zmax,rmax,rr,xx,zz,theta,ddx,ddz,ddr
        real a11,a12,a21,a22,t0,drdt,s2,u2,r2,th
        real hs,wminb,wmaxf,wmin1f,wmax1b,forw,bckw,ss,s2u2
        integer it,ir,ix,iz,ir1,ir2,nrconst,nthalf,itemp
        integer istopflag
        real drrem,drrest,rhere,rw,cmax,TINY,RTINY,CFL

        real dx1,dx2
 	real bilint
        bilint(dx1,dx2,a11,a21,a12,a22) =
     &     (1-dx1)*(1-dx2)*a11+dx1*(1-dx2)*a21+
     &               (1-dx1)*dx2*a12+dx1*dx2*a22

C nr       = number of radius increments
C ntheta  = number of angle increments
C            usually nr=nz
C            usually ntheta=nx
C
        xmax=o_x+(nx-1)*d_x
        zmax=o_z+(nz-1)*d_z

C Check if shot coordinates are outside model
        if( (x_shot.lt.o_x.or.x_shot.gt.xmax) .or. 
     &  (z_shot.lt.o_z.or.z_shot.gt.zmax) ) then
          write(*,*) 'Shot coordinates are outside grid model'
          return
        endif
C Initialize constants
        PI = 4.*atan(1.)
        PI2=PI*0.5
        eps=0.001
        o_radius = 0.
        TINY=1.0e-3
        RTINY=1./TINY
C        CFL=0.98
        CFL=.98*cflfudge
C Find the most distant corner from the shot location, that is the max. radius
        xmax=max(abs(x_shot-o_x),abs(xmax-x_shot))
        zmax=max(abs(z_shot-o_z),abs(zmax-z_shot))
        rmax=sqrt(xmax*xmax+zmax*zmax)
        dr =(rmax-o_radius) / (nr-2)
        rdr=1./dr

C###########################################################################
C Map to cylindrical coordinates
C otheta = initial angle, dtheta = angle increment
        otheta = -(90.-eps)*PI / 180.
        dtheta = 2.*abs(otheta)/(ntheta-2)
        otheta = otheta - dtheta
        rdt=1./dtheta
C Loop in Radius and Theta and express the slowness in cylindrical coordinates
        xmax=o_x+(nx-2)*d_x
        zmax=o_z+(nz-2)*d_z
        do ir=1,nr
          rr=o_radius+(ir-1)*dr
          do it=1,ntheta+2
            theta=otheta+(it-1)*dtheta
            xx=rr*sin(theta)+x_shot
            zz=rr*cos(theta)+z_shot

C xx and zz stay within the boundaries (o_x,xmax) respectively (o_z,zmax)
            xx=max(o_x,xx)
            xx=min(xmax,xx)
            zz=max(o_z,zz)
            zz=min(zmax,zz)
C Find index in the (X,Z) grid
            ix=int((xx-o_x)/d_x)+1
            iz=int((zz-o_z)/d_z)+1

            ddx=(xx-o_x)/d_x-ix+1
            ddz=(zz-o_z)/d_z-iz+1
	    ix=min(ix,nx-1)
	    iz=min(iz,nz-1)

            a11=slow(iz,ix)
            a21=slow(iz+1,ix)
            a12=slow(iz,ix+1)
            a22=slow(iz+1,ix+1)
            slow_rad(it,ir)=bilint(ddz,ddx,a11,a21,a12,a22)
          enddo
        enddo

C############################################################################
C Finite differences calculation of travel times in cylindrical coordinates
C
C Initialize U and T; assume constant velocity in first nrconst radii
        t0=0.
        nrconst=3
        nthalf=ntheta/2
        epsilon=1.e-20
        do ir=1,nrconst
          rr=o_radius+(ir-1)*dr
          do it=0,ntheta+1
            t(it,ir)=t0+slow_rad(2,1)*rr
          enddo
        enddo
        do it=0,ntheta+1
          ttemp(it)=t(it,nrconst)
        enddo
C Give minuscule flow to force rays outward at the boundaries
        ir1=1
        ir2=2
        do it=0,ntheta+1
          if(it.lt.nthalf) then
            u(it,ir1)=-epsilon
          else
            u(it,ir1)=epsilon
          endif
          w(it,ir1)=slow_rad(2,1)
        enddo

C Calculate derivative of flux function at fixed R level
C rr    = current radius
C u     = dt/dtheta
C w     = dt/dr
C fx    = dw

        do ir=nrconst,nr-1

C radius is calculated for the next level
          rr=o_radius+ir*dr
          do it=1,ntheta+2
            slowtemp(it)=slow_rad(it,ir)
          enddo
          drrem=0.
          istopflag=0
100       continue
          rhere=rr-dr+drrem

C############################################################################
C Implement the stability condition by varying the size of dr
C (Smart piece of code from Dave Hale)
C TINY accounts for avoiding w=0
C When u=0 everything is fine, dr is sufficient
C When u/(r*w) > 1 we have potential instability;
C (this happens when the plane wave propagates at more
C then 45 degrees from local radial direction for ds=dr)
C In this case divide r*dtheta by the ratio u/(r*w)  
C Start CFL test
          cmax=TINY
          do it=0,ntheta+1
            rw=ABS(w(it,ir1)*rhere)
            if(rw.lt.TINY*ABS(u(it,ir1))) then
              cmax=RTINY
            else
              cmax=max( cmax,abs(u(it,ir1)/rw) )
            endif
          enddo
          ddr=min(dr,CFL/cmax*rhere*dtheta)
C ddr is the next dr increment          
C End CFL test
C############################################################################
          drrem=drrem+ddr
          drrest=dr-drrem
          if(drrest.le.0) then
            ddr=drrest+ddr
            drrest=0.
            drrem=dr
            istopflag=1
          endif
          r2=(rhere+ddr)*(rhere+ddr)
          do it=1,ntheta
            ss=slowtemp(it+1)
            hs=0.5*(1.+sign( 1.,u(it,ir1) ))

C +F[max(u_j,0)]
            wmaxf=hs*w(it,ir1)+(1.-hs)*ss
C -F[min(u_j,0)]
            wminb=(1.-hs)*w(it,ir1)+hs*ss
C +F[min(u_j+1,0)]
            hs=0.5*(1.+sign( 1.,u(it+1,ir1) ))
            wmin1f=(1.-hs)*w(it+1,ir1)+hs*ss
C -F[max(u_j-1,0)]
            hs=0.5*(1.+sign( 1.,u(it-1,ir1) ))
            wmax1b=hs*w(it-1,ir1)+(1.-hs)*ss

            forw=wmaxf+wmin1f
            bckw=wmax1b+wminb
            fx(it)=forw-bckw
          enddo

C BOUNDARIES: Assume outgoing rays
C Left Side
          fx(0)=w(1,ir1)-w(0,ir1)
C Right Side
          fx(ntheta+1)=w(ntheta+1,ir1)-w(ntheta,ir1)
C Interpolate the velocity for the new radius 
          do it=1,ntheta+2
            ss=(slow_rad(it,ir)*drrest+slow_rad(it,ir+1)*drrem)*rdr
            slowtemp(it)=ss
          enddo
C Update the next level
          drdt=ddr*rdt
          do it=0,ntheta+1
            u(it,ir2)=u(it,ir1)+drdt*fx(it)
            ss=slowtemp(it+1)
            s2=ss*ss
            u2=u(it,ir2)*u(it,ir2)
	    s2u2=s2-u2/r2
	    s2u2=max(s2u2,0.)
	    w(it,ir2)=sqrt(s2u2)
            ttemp(it)=ttemp(it)+0.5*ddr*(w(it,ir1)+w(it,ir2))
          enddo
	  itemp=ir1
	  ir1=ir2
	  ir2=itemp
          if(istopflag.ne.1) go to 100
          do it=0,ntheta+1
            t(it,ir+1)=ttemp(it)
          enddo
        enddo

C############################################################################
C Map to cartezian coordinates
        do iz=1,nzo
          zz=ozo+(iz-1)*dzo-z_shot
          do ix=1,nxo
            xx=oxo+(ix-1)*dxo-x_shot
            rr=sqrt(xx*xx+zz*zz)
            if(abs(zz).lt.epsilon) then
              th=sign(1.,xx)*PI2
            else
              th=atan2(xx,zz)
            endif
            it=int((th-otheta)*rdt)+1
            it=max(1,it)
            it=min(ntheta+1,it)
            ir=int((rr-o_radius)*rdr)+1
            ir=max(1,ir)
            ir=min(nr-1,ir)

            ddr=(rr-o_radius)*rdr-ir+1
            ddt=(th-otheta)*rdt-it+1
            a11=t(it-1,ir)
            a21=t(it,ir)
            a12=t(it-1,ir+1)
            a22=t(it,ir+1)
            travel_time(iz,ix)=bilint(ddt,ddr,a11,a21,a12,a22)
          enddo
        enddo
        return
        end

C###############################################################################
C  dx1   = Z distance to upper left corner
C  dx2   = X distance to upper left corner
C  a11   = Upper Left
C  a21   = Lower Left
C  a12   = Upper Right
C  a22   = Lower Right
C
C Interpolation is done using the function
C f(x,z)=A*x*z+B*x+C*z+D
C and solving the system f(0,0)=a11
C                        f(0,1)=a12
C                        f(1,0)=a21
C                        f(1,1)=a22
C###############################################################################
C        real function bilint(dx1,dx2,a11,a21,a12,a22)
C        real dx1,dx2,a11,a21,a12,a22
C        bilint=(1-dx1)*(1-dx2)*a11+dx1*(1-dx2)*a21+
C     &               (1-dx1)*dx2*a12+dx1*dx2*a22
C        return
C        end


