########################################################################  
#  Usage:    
#  MTTmaps <slowmodel.H x_shot= mode=0 >maps.H
#
#  Band-limited maximum-energy Green's function maps
#
#  Author: Dave Nichols
#
# Input
# =====
# slow(nz,nx) = slowness model
#
# Output
# ======
# maps(nzo,nxo,3,n_shot) = output Green's function maps, tt/amp/phase
#
# Default parameters calculate one shot at user specified output location.
# x_shot must be specified! The output grid will be the same as the input grid.
#
# When calculating multiple maps you may want to specify the other
# parameters that control the output grid relative to each shot.
# There are two modes, if mode=0 the output grid is absolute.
# If mode=1 the output grid is relative to the shot location.
#
# Parameters for shot locations
# ======================================
# z_shot=0                = shot Z coordinate
# x_shot                  = first shot X coordinate
# dx_shot=dxo	  	  = increment in shot X coordinate
# n_shot=1		  = number of shots
#
# Parameters for output rectangular grid
# ======================================
# mode=0	grid is stationary the shot moves across the grid
#               the second output axis coordinates are absolute.
# oxo=o2         	  = initial surface coordinate
# dxo=d2		  = increment in surface coordinate
# nxo=n2                  = number of surface coordinates
# ozo=o1,dzo=d1           = first depth level and depth increments
# nzo=n1                  = number of depth levels on the grid
#
# mode=1	grid moves with the shot, the second output axis
#		coordinates are relative to the shot location.
# oxo=(o2-x_shot)         = first offset from shot coordinate 
# dxo=d2		  = increment in offset
# nxo=n2                  = number of offsets
# ozo=o1,dzo=d1           = first depth level and depth increments
# nzo=n1                  = number of depth levels on the grid
#
# Parameters for polar computational grid
# =======================================
# n_theta=nxo             = number of angles for the polar grid
# nr=nzo                  = number of radius increments for the polar grid
#
# fmin=10		  = minimum frequency in estimation band
# fmax=60		  = maximum frequency in estimation band
# nf=16			  = number of frequencies to use in estimation.
# dt=.008		  = sampling to use for initial traveltime estimation
#
#%
#  source= ./cube/seis/MTTmaps.rs
#

########################################################################
#
# Author, Dave Nichols (SEP) March 19 1994
#

integer nx,nz,nr,ntheta
integer nxo,nzo,nshot,mode,nf
real    xshot,zshot,ox,dx,oz,dz
real    oxo,dxo,ozo,dzo,dxshot
real    or,dr,otheta,dtheta,fmin,fmax,dt
real 	rmax,zmax,apmax,v,thetamax
#
#      input slowness map parameters
#
from history:   integer n1:nz, n2:nx
from history:   real o1:oz, o2:ox, d1:dz, d2:dx
from par:       integer mode=0
#
#	shot coordinates
#
from par:       real x_shot:xshot, z_shot:zshot=0.
from par:	integer n_shot:nshot=1
from par:	real dx_shot:dxshot=dxo

#
#	sampling of output grid
#
from par:	integer nzo=nz,nxo=nx
from par:	real ozo=oz,dzo=dz,dxo=dx
if( mode == 0 ){
        from par:	real oxo=ox
}else{
        from par:	real oxo=(ox-xshot)
}

#
#	frequency sampling and rough time sampling
#
from par: integer nf=16
from par: real fmin=10.,fmax=60.,dt=.008

#
#	size of polar grid
#

or = 0
from par:       real v=2000. 
# calculate a reasonable dr ( 3 points per min wavelength )
dr = ( (v/fmax) / 3. )
if( mode == 0 ){
   apmax = max( abs(xshot-oxo), abs(oxo+(nxo-1)*dxo-xshot) )
}else{
   apmax = max( abs(oxo), abs(oxo+(nxo-1)*dxo) )
}
zmax = ozo + (nzo-1)*dzo
rmax=sqrt(zmax**2 + apmax**2 )
nr = rmax/dr +2

from par: integer n_theta:ntheta=nxo
thetamax = 3.141592654*0.99/2.
otheta = -thetamax
dtheta = (2.*thetamax)/real(ntheta-1)

allocate:       real poltime(ntheta,nr) 
allocate:       real polamp(ntheta,nr) 
allocate:       real polphase(ntheta,nr) 
allocate:       real slow(nz,nx), polslow(ntheta,nr)
allocate:       real omega(nf)

subroutine mttmap(xshot,zshot,nx,ox,dx,nz,oz,dz,
	slow,polslow,poltime,polamp,polphase,
	nxo,nzo,oxo,dxo,ozo,dzo,nshot,dxshot,mode,
        ntheta,nr,or,dr,otheta,dtheta,nf,fmin,fmax,dt,v,omega)

integer nx,nz,nr,ntheta
integer nxo,nzo,nshot,mode,nf
real    oxo,dxo,ozo,dzo,dxshot
real xshot,zshot,ox,dx,oz,dz
real    or,dr,otheta,dtheta,fmin,fmax,dt,v

real poltime(ntheta,nr) 
real polamp(ntheta,nr) 
real polphase(ntheta,nr) 
real slow(nz,nx), polslow(ntheta,nr)
real omega(nf) 

real ooxo,dfreq,pi/3.1415926536/
integer input,output,infd,outfd,esize,i,n3o,irad0
integer nsamp,cosamp,polout

from par: integer cosamp=0
from par: boolean polout=0

infd=input()
outfd=output()
esize=4

n3o = 3

if( polout == 0 ){
to history: integer n1:nzo, n2:nxo, n3:n3o, n4:nshot
to history: real o1:ozo, o2:oxo, o4:xshot
to history: real d1:dzo, d2:dxo, d4:dxshot
}else{
to history: integer n1:ntheta, n2:nr, n3:n3o, n4:nshot
to history: real o1:otheta, o2:or, o4:xshot
to history: real d1:dtheta, d2:dr, d4:dxshot
}

call hclose()

dfreq= (fmax-fmin)/(nf-1)
do i=1,nf{ omega(i) = (fmin + dfreq*(i-1))*2.*pi }
nsamp = max( nint(.25/(dfreq * dt)) , 3 )

irad0 = (v/(fmin*2)-or)/dr + 1
irad0 = max(irad0,4)


# read the slowness field

call sreed('in',slow,nz*nx*esize)

do i=1,nshot{

     # Interpolate slowness to polar grid
     call rect2pol( slow, nx, ox, dx, nz, oz, dz ,
                     polslow, ntheta, otheta, dtheta, nr, or, dr ,
                     xshot, zshot )

     # compute Green's function on polar grid
     call Polgreens( ntheta,dtheta,otheta,nr,or,dr,irad0,
         polslow,poltime,polamp,polphase,
         omega,nf,dt,nsamp,cosamp )

     if( mode == 0 ){ 
        # output grid is stationary
        ooxo = oxo
     }else{
        # output grid is relative to shot
        ooxo = xshot + oxo
     }

     if( polout == 0 ){
         # Interpolate maps to rect grid and output

         call rectout( poltime, ntheta,dtheta,otheta,nr,or,dr,
	 	     ooxo,nxo,dxo,ozo,dzo,nzo, zshot,xshot )
         call rectout( polamp, ntheta,dtheta,otheta,nr,or,dr,
	 	     ooxo,nxo,dxo,ozo,dzo,nzo, zshot,xshot )
         call rectout( polphase, ntheta,dtheta,otheta,nr,or,dr,
	 	     ooxo,nxo,dxo,ozo,dzo,nzo, zshot,xshot  )
      }else{
	 call srite('out',poltime,ntheta*nr*4)
	 call srite('out',polamp,ntheta*nr*4)
	 call srite('out',polphase,ntheta*nr*4)
      }

     # increment shot coord.
     xshot  = xshot + dxshot

}

#call exit(0)
return
end
