########################################################################  
#  Usage:    
#  <slowmodel.H TTmap x_shot= mode=0 >traveltimemap.H
#
#  Finite Differences Travel Time Map
#  Author: Mihai Popovici (Jos Van Trier's algorithm)
#
# Input
# =====
# slow(nz,nx) = slowness model
#
# Output
# ======
# travel_time(nzo,nxo,n_shot) = output travel time map[s]
#
# Default parameters calculate one shot at user specified output location.
# x_shot must be specified! The output grid will be the same as the input grid.
#
# When calculating multiple maps you may want to specify the other
# parameters that control the output grid relative to each shot.
# There are two modes, if mode=0 the output grid is absolute.
# If mode=1 the output grid is relative to the shot location.
#
# Parameters for shot locations
# ======================================
# z_shot=0                = shot Z coordinate
# x_shot                  = first shot X coordinate
# dx_shot=dxo	  	  = increment in shot X coordinate
# n_shot=1		  = number of shots
#
# Parameters for output rectangular grid
# ======================================
# mode=0	grid is stationary the shot moves across the grid
#               the second output axis coordinates are absolute.
# oxo=o2         	  = initial surface coordinate
# dxo=d2		  = increment in surface coordinate
# nxo=n2                  = number of surface coordinates
# ozo=o1,dzo=d1           = first depth level and depth increments
# nzo=n1                  = number of depth levels on the grid
#
# mode=1	grid moves with the shot, the second output axis
#		coordinates are relative to the shot location.
# oxo=(o2-x_shot)         = first offset from shot coordinate 
# dxo=d2		  = increment in offset
# nxo=n2                  = number of offsets
# ozo=o1,dzo=d1           = first depth level and depth increments
# nzo=n1                  = number of depth levels on the grid
#
# Parameters for polar computational grid
# =======================================
# n_theta=nxo              = number of angles for the polar grid
# nr=nzo                   = number of radius increments for the polar grid
#
# cflfudge=1.		  fudge the CFL stability condition
#                         1.== strict.    >1. == less strict (possibly unstable)
#
#  When raytracing is unstable increase the ratio nr/n_theta
#  (Increase nr or decrease n_theta).
#
#
#  source= ./cube/seis/TTmaps.rs
#

########################################################################
#%
# Mihai, Implemented CFL stability condition JULY/91
#
# Modified, Dave Nichols (SEP) Feb 17 1994
#    Added ability to output multiple surface locations
#    Changed into a ratfor file.
#    Added cfl_fudge parameter
#

        integer nx,nz,nr,n_theta
        integer nxo,nzo,n_shot,mode
        real    x_shot,z_shot,o_x,d_x,o_z,d_z
        real    oxo,dxo,ozo,dzo,dx_shot
#
#      input slowness map parameters
#
        from history:   integer n1:nz, n2:nx
        from history:   real o1:o_z, o2:o_x, d1:d_z, d2:d_x
        from par:       integer mode=0
#
#	shot coordinates
#
        from par:       real x_shot, z_shot:z_shot=0.
        from par:	integer n_shot=1
        from par:	real dx_shot=dxo

#
#	sampling of output grid
#
        from par:	integer nzo=nz,nxo=nx
        from par:	real ozo=o_z,dzo=d_z,dxo=d_x
        if( mode == 0 ){
        from par:	real oxo=o_x
        }else{
        from par:	real oxo=(o_x-x_shot)
        }

#
#	size of polar grid
#
        from par:       integer nr=nzo, n_theta=nxo

        allocate:       real t(n_theta+2,nr) 
	allocate:       real u(n_theta+2,2), w(n_theta+2,2)
        allocate:       real slow(nz,nx), slow_rad(n_theta+2,nr)
        allocate:       real fx(n_theta+2), travel_time(nzo,nxo)
        allocate:       real slowtemp(n_theta+2), ttemp(n_theta+2)

        subroutine callfd(x_shot,z_shot,nx,o_x,d_x,nz,o_z,d_z, n_theta,nr, slow,travel_time,t,u,w,fx,slow_rad,slowtemp,ttemp, nxo,nzo,oxo,dxo,ozo,dzo,n_shot,dx_shot,mode)
        integer nx,nz,nr,n_theta
        integer nxo,nzo,n_shot,mode
        real    oxo,dxo,ozo,dzo,dx_shot
        real t(0:n_theta+1,nr),u(0:n_theta+1,2),w(0:n_theta+1,2)
        real slow(nz,nx), slow_rad(n_theta+2,nr)
        real fx(0:n_theta+1)
        real travel_time(nz,nx)
        real slowtemp(n_theta+2),ttemp(0:n_theta+1)
        real x_shot,z_shot,o_x,d_x,o_z,d_z,ooxo
        integer input,output,infd,outfd,esize,i
        real cflfudge

        from par: real cflfudge=1.

        infd=input()
        outfd=output()
        esize=4
        call putch('n1','i',nzo)
        call putch('n2','i',nxo)
        call putch('n3','i',n_shot)
        call putch('o1','f',ozo)
        call putch('o2','f',oxo)
        call putch('o3','f',x_shot)
        call putch('d1','f',dzo)
        call putch('d2','f',dxo)
        call putch('d3','f',dx_shot)
        call hclose()
        call reed(infd,slow,nz*nx*esize)

        do i=1,n_shot{
      
        if( mode == 0 ){ 
	    # output grid is stationary
            ooxo = oxo
   	}else{
	    # output grid is relative to shot
            ooxo = x_shot + oxo
	}

        call FDTraTim(x_shot,z_shot,nx,o_x,d_x,nz,o_z,d_z, n_theta,nr,
		slow,travel_time,t,u,w,fx,slow_rad,slowtemp,ttemp, 
		nxo,nzo,ooxo,dxo,ozo,dzo,cflfudge)

        call rite(outfd,travel_time,nzo*nxo*esize)

	x_shot  = x_shot + dx_shot

        }

        return
        end




